@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: sfsasl.c,v 8.17.4.14 2001/05/03 17:24:16 gshapiro Exp $";
+static char id[] = "@(#)$Id: sfsasl.c,v 8.17.4.15 2001/07/11 17:37:07 gshapiro Exp $";
 #endif /* ! lint */
 
 #if SFIO
@@ -168,18 +168,19 @@ sfdcsasl(fin, fout, conn)
 # include "sfsasl.h"
 #  include <openssl/err.h>
 
-static ssize_t
 # if SFIO
+static ssize_t
 tls_read(f, buf, size, disc)
 	Sfio_t *f;
 	Void_t *buf;
 	size_t size;
 	Sfdisc_t *disc;
 # else /* SFIO */
+static int
 tls_read(disc, buf, size)
 	void *disc;
-	void *buf;
-	size_t size;
+	char *buf;
+	int size;
 # endif /* SFIO */
 {
 	int r;
@@ -226,18 +227,19 @@ tls_read(disc, buf, size)
 	return r;
 }
 
-static ssize_t
 # if SFIO
+static ssize_t
 tls_write(f, buf, size, disc)
 	Sfio_t *f;
 	const Void_t *buf;
 	size_t size;
 	Sfdisc_t *disc;
 # else /* SFIO */
+static int
 tls_write(disc, buf, size)
 	void *disc;
-	const void *buf;
-	size_t size;
+	const char *buf;
+	int size;
 # endif /* SFIO */
 {
 	int r;
